package com.persagy.dmp.admin;

import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JpaUtils;
import com.persagy.dmp.common.SpringUtils;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.rwd.funcid.parse.parser.InfoCodeParserFactory;
import com.querydsl.jpa.impl.JPAQueryFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.mgt.SecurityManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.Bean;

import javax.persistence.EntityManager;

/**
 * @author: yaoll
 * @date: 2020-12-29
 * @verison: 1.0
 */
@Slf4j
@SpringBootApplication
@EnableDiscoveryClient
public class AdminApp {

	public static void main(String[] args) {
		SpringApplication app = new SpringApplication(AdminApp.class);
		ConfigurableApplicationContext ctx = app.run(args);

		SecurityManager securityManager = ctx.getBean(SecurityManager.class);
		SecurityUtils.setSecurityManager(securityManager);

	}

	@Bean
	public InfoCodeParserFactory infoCodeParserFactory() {
		return new InfoCodeParserFactory();
	}

	@Bean
	public JPAQueryFactory jpaQueryFactory(EntityManager entityManager) {
		return new JPAQueryFactory(entityManager);
	}

	@Bean
	public CriteriaUtils criteriaUtils(@Autowired JPAQueryFactory jpaQueryFactory) {
		return new CriteriaUtils(jpaQueryFactory);
	}

	@Bean
	public JpaUtils jpaUtils(@Autowired EntityManager entityManager) {
		return new JpaUtils(entityManager);
	}

	@Bean
	public HttpUtils httpUtils() {
		return new HttpUtils();
	}

	@Bean
	public SpringUtils springUtils() {
		return new SpringUtils();
	}

}
