package com.persagy.dmp.rwd.edit.service;

import com.persagy.common.web.MapResponse;
import com.persagy.dmp.rwd.edit.config.shiro.UserModel;
import com.persagy.dmp.rwd.edit.entity.AdminUser;
import com.persagy.dmp.rwd.edit.repository.AdminPermissionRepository;
import com.persagy.dmp.rwd.edit.repository.AdminUserRepository;
import com.querydsl.core.BooleanBuilder;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.data.redis.core.StringRedisTemplate;

import java.util.*;

import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
class UserServiceTest {

    @Mock
    private AdminUserRepository mockUserRepository;
    @Mock
    private AdminPermissionRepository mockPermissionRepository;
    @Mock
    private StringRedisTemplate mockRedis;

    @InjectMocks
    private UserService userServiceUnderTest;

    @Test
    void testLogin() {
        // Setup
        final UserModel userModel = new UserModel();
        userModel.setId(0);
        userModel.setToken("token");
        userModel.setUsername("username");
        userModel.setPassword("password");
        userModel.setPermissions(Arrays.asList("value"));

        // Configure AdminUserRepository.findOne(...).
        final AdminUser adminUser1 = new AdminUser();
        adminUser1.setId(0);
        adminUser1.setUsername("username");
        adminUser1.setPassword("password");
        adminUser1.setRealname("realname");
        adminUser1.setPhone("phone");
        adminUser1.setRemark("remark");
        adminUser1.setStatus(0);
        adminUser1.setCreateUser(0);
        adminUser1.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        adminUser1.setUpdateUser(0);
        final Optional<AdminUser> adminUser = Optional.of(adminUser1);
        when(mockUserRepository.findOne(new BooleanBuilder())).thenReturn(adminUser);

        when(mockRedis.opsForValue()).thenReturn(null);

        // Run the test
        final MapResponse result = userServiceUnderTest.login(userModel);

        // Verify the results
    }

    @Test
    void testLogin_AdminUserRepositoryReturnsAbsent() {
        // Setup
        final UserModel userModel = new UserModel();
        userModel.setId(0);
        userModel.setToken("token");
        userModel.setUsername("username");
        userModel.setPassword("password");
        userModel.setPermissions(Arrays.asList("value"));

        when(mockUserRepository.findOne(new BooleanBuilder())).thenReturn(Optional.empty());
        when(mockRedis.opsForValue()).thenReturn(null);

        // Run the test
        final MapResponse result = userServiceUnderTest.login(userModel);

        // Verify the results
    }

    @Test
    void testLogout() {
        // Setup
        when(mockRedis.opsForValue()).thenReturn(null);
        when(mockRedis.delete("key")).thenReturn(false);

        // Run the test
        final MapResponse result = userServiceUnderTest.logout();

        // Verify the results
    }

    @Test
    void testLogout_StringRedisTemplateDeleteReturnsNull() {
        // Setup
        when(mockRedis.opsForValue()).thenReturn(null);
        when(mockRedis.delete("key")).thenReturn(null);

        // Run the test
        final MapResponse result = userServiceUnderTest.logout();

        // Verify the results
    }

    @Test
    void testUserInfo() {
        // Setup

        // Configure AdminUserRepository.findOne(...).
        final AdminUser adminUser1 = new AdminUser();
        adminUser1.setId(0);
        adminUser1.setUsername("username");
        adminUser1.setPassword("password");
        adminUser1.setRealname("realname");
        adminUser1.setPhone("phone");
        adminUser1.setRemark("remark");
        adminUser1.setStatus(0);
        adminUser1.setCreateUser(0);
        adminUser1.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        adminUser1.setUpdateUser(0);
        final Optional<AdminUser> adminUser = Optional.of(adminUser1);
        when(mockUserRepository.findOne(new BooleanBuilder())).thenReturn(adminUser);

        when(mockPermissionRepository.queryForUser(0)).thenReturn(Arrays.asList("value"));

        // Run the test
        final MapResponse result = userServiceUnderTest.userInfo();

        // Verify the results
    }

    @Test
    void testUserInfo_AdminUserRepositoryReturnsAbsent() {
        // Setup
        when(mockUserRepository.findOne(new BooleanBuilder())).thenReturn(Optional.empty());
        when(mockPermissionRepository.queryForUser(0)).thenReturn(Arrays.asList("value"));

        // Run the test
        final MapResponse result = userServiceUnderTest.userInfo();

        // Verify the results
    }

    @Test
    void testUserInfo_AdminPermissionRepositoryReturnsNoItems() {
        // Setup

        // Configure AdminUserRepository.findOne(...).
        final AdminUser adminUser1 = new AdminUser();
        adminUser1.setId(0);
        adminUser1.setUsername("username");
        adminUser1.setPassword("password");
        adminUser1.setRealname("realname");
        adminUser1.setPhone("phone");
        adminUser1.setRemark("remark");
        adminUser1.setStatus(0);
        adminUser1.setCreateUser(0);
        adminUser1.setCreateTime(new GregorianCalendar(2020, Calendar.JANUARY, 1).getTime());
        adminUser1.setUpdateUser(0);
        final Optional<AdminUser> adminUser = Optional.of(adminUser1);
        when(mockUserRepository.findOne(new BooleanBuilder())).thenReturn(adminUser);

        when(mockPermissionRepository.queryForUser(0)).thenReturn(Collections.emptyList());

        // Run the test
        final MapResponse result = userServiceUnderTest.userInfo();

        // Verify the results
    }
}
